// (C) Copyright 1998 D.C.Meintjes - dirk@kivtronics.co.za
//
// To see the exposed properties and callable methods, search for the line
// "USER INFO HERE". Everything before that line is for internal use.
//
//---------------------------------------------------------------------------
#ifndef ModemH
#define ModemH

#ifndef __FROMMODEM__
#include "uart.h"
#endif

#include "ModemProgress.h"
#include "DialOptions.h"
#include "modemselect.h"
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#define TAPI_CURRENT_VERSION	0x00010004
#include <tapi.h>
//---------------------------------------------------------------------------
// this structure holds the modem info returned by TAPI
typedef struct modem_info_tag
{
	VARSTRING vs;
	HANDLE hComm;
	char szDeviceName[255];
} MODEM_INFO, FAR *LPMODEM_INFO;

typedef struct device_config_tag
{
	VARSTRING vs;
	DWORD       dwSize;
	DWORD       dwVersion;
	WORD        fwOptions;
	WORD        wWaitBong;
	COMMCONFIG  commconfig;
} DEVICECONFIG, FAR *LPDEVICECONFIG;

#define TERMINAL_PRE	1
#define TERMINAL_POST	2
#define	MANUAL_DIAL		4
#define	LAUNCH_LIGHTS	8
//---------------------------------------------------------------------------
// enumerated types
enum TDCModemStatus	{msError, msIdle, msRinging, msAnswering, msConnecting,
					 msConnected, msWaitReply, msDialling, msBusy, msDisconnected,
					 msLineClose, msOk};
enum TDCModemError {msLineError};
enum TDCModemResult {mSuccess, mCancelled, mBusy, mError};
enum TDCModemOption {moDefault, moYes, moNo};
//---------------------------------------------------------------------------
// Custom events
typedef void __fastcall (__closure *TStatusChangeEvent)(TObject *Sender, TDCModemStatus Status, String StatStr);
typedef void __fastcall (__closure *TModemConnectEvent)(TObject *Sender, TDCModemStatus Status);
typedef void __fastcall (__closure *TModemDisconnectEvent)(TObject *Sender, TDCModemStatus Status);
typedef void __fastcall (__closure *TDeviceChangeEvent)(TObject *Sender, DWORD NewDevice);

//*****************************************************************************
// classdeclaration form TModem component
//

class PACKAGE TModem : public TComponent
{
private:
// these variables are used internally and are not property storage holders
	HLINEAPP LineApp;							// handle to TAPI DLL connection
	HCALL Call;									// handle to TAPI call
	LINECALLPARAMS CallParams;					// parameters of call
	HANDLE hCommDevice;							// communications file handle
	bool Connected;								// holds state of modem connection
	bool DialogOpen;
	bool Listening;
	DWORD DeviceCount;							// number of modems installed
	DWORD tapiVersion;							// version of TAPI we'll use
	DWORD DeviceID;								// ID of selected modem
	DWORD DeviceIndex;							// index of current device

	void __fastcall BuildDeviceList();			// builds list of attached modems

// property storage variables
	TDCModemStatus	FStatus;					// holds our line status
	TUart *FUart;								// ptr to UART component
	TDCModemOption FManualDial;					// controls manual dialling
	TDCModemOption FTerminalBefore;				// show TERM Before dialling
	TDCModemOption FTerminalAfter;				// show TERM After dialling
	TDCModemOption FTrayMonitor;				// show ICONTRAY monitor
	bool FUseDialDlg;							// TRUE to use dialling dialog
	bool FShowDetails;							// TRUE to show dialling details
	System::AnsiString FAreaCode;				// holds area code to dial
	System::AnsiString FOutsideLine;			// holds area code to dial
	System::AnsiString FPhoneNo;				// phone number to dial
	DWORD FCountryID;							// ID of country to dial to
	DWORD FLocationID;							// ID of location dialling from
	bool FUseDialProp;							// TRUE to use system dialling props

// event handler storage
	TStatusChangeEvent	FStatusChangeEvent;	// called when status changes
	TModemConnectEvent		FConnectEvent;		// called when modem connects
	TModemDisconnectEvent	FDisconnectEvent;	// called when call is disconnected
	TDeviceChangeEvent	FDeviceChangeEvent;	// called when user select new

// routines used internally
	void __fastcall GetCurrentCountryID();
	void __fastcall GetCurrentLocationID();

protected:

public:
// these events are used internally but made public so the dialogs used by
// this component can access them. The user should NOT call these functions
// or use these variables directly.
	bool ProgressDlgOpen;
	TStringList *ModemList;
	DWORD RingsToAnswer;
	TModemProgressDlg *ProgressDlg;
	String TelNo, FriendlyNo;
	HLINE Line;									// handle to TAPI line
	TDCModemResult DialResult;

	__fastcall TModem(TComponent* Owner);
	__fastcall ~TModem();
	void __fastcall Shutdown();
	void __fastcall Startup();
	void __fastcall SetStatus(TDCModemStatus stat, String s);
	void __fastcall SetCallProperties();
	void __fastcall FreeHandle(void);
// property getters/setters
	HANDLE __fastcall GetHandle(void);
	void __fastcall SetDevice(DWORD dev);
	System::AnsiString __fastcall GetDevice();
	System::AnsiString __fastcall GetCountry();
	System::AnsiString __fastcall GetLocation();

// USER INFO HERE - See top of file	
// methods callable by users
	TDCModemResult __fastcall Dial(String No);			// dials a number
	TDCModemResult __fastcall Dial();					// dials PhoneNo property
	void __fastcall Hangup();							// terminates call
	bool __fastcall Listen(DWORD Rings = 3);			// waits for incoming call
	void __fastcall StopListen();						// stops waiting for call
	bool __fastcall ShowConfigureDialog();				// modem configuration
	void __fastcall UserSelectModem();					// uhm
	bool __fastcall ShowDialDialog();					// sets dial location properties
	bool __fastcall ShowNumberDialog();					// gathers phone no from user
	int __fastcall FillDeviceList(TStrings *s);			// fills list with device names
	int __fastcall FillCountryList(TStrings *s);		// fills list with country names
	int __fastcall FillLocationList(TStrings *s);		// fills list with location names
	System::AnsiString __fastcall GetDeviceName(int device = -1);
	System::AnsiString __fastcall GetCountryName(DWORD &CountryID);
	System::AnsiString __fastcall GetLocationName(DWORD LocationID);
	void __fastcall DrawDeviceIcon(TImage *image, TColor bgcolor, int dev = -1);
	DWORD __fastcall GetCountryCode(DWORD CountryID);
	System::AnsiString __fastcall GetDiallableNumber(DWORD CountryID,
													System::AnsiString AreaCode,
													System::AnsiString PhoneNo);

// runtime properties
	__property TDCModemStatus Status={read=FStatus};
	__property HANDLE Handle={read=GetHandle};
	__property DWORD Device={read=DeviceIndex,write=SetDevice};
	__property DWORD ModemCount={read=DeviceCount};
	__property DWORD Country={read=FCountryID,write=FCountryID};
	__property DWORD Location={read=FLocationID, write=FLocationID};
	__property System::AnsiString CountryName={read=GetCountry};
	__property System::AnsiString LocationName={read=GetLocation};
    __property System::AnsiString DeviceName={read=GetDevice};

__published:

// designtime properties
	__property TUart* Uart={read=FUart,write=FUart,default=NULL};
	__property TDCModemOption ManualDial={read=FManualDial,write=FManualDial,default=moDefault};
	__property TDCModemOption TermAfter={read=FTerminalAfter, write=FTerminalAfter, default=moDefault};
	__property TDCModemOption TermBefore={read=FTerminalBefore, write=FTerminalBefore, default=moDefault};
	__property TDCModemOption TrayMonitor={read=FTrayMonitor,write=FTrayMonitor,default=moDefault};
	__property bool UseDialDialog={read=FUseDialDlg,write=FUseDialDlg,default=true};
	__property bool ShowDialDetails={read=FShowDetails,write=FShowDetails,default=-true};
	__property bool UseDialProp={read=FUseDialProp,write=FUseDialProp,default=true};
	__property System::AnsiString AreaCode={read=FAreaCode,write=FAreaCode,nodefault};
	__property System::AnsiString OutsideLine={read=FOutsideLine,write=FOutsideLine,nodefault};
	__property System::AnsiString PhoneNo={read=FPhoneNo,write=FPhoneNo,nodefault};

// events
	__property TStatusChangeEvent OnStatusChange={read=FStatusChangeEvent,write=FStatusChangeEvent};
	__property TModemConnectEvent OnConnect={read=FConnectEvent,write=FConnectEvent};
	__property TModemDisconnectEvent OnDisconnect={read=FDisconnectEvent,write=FDisconnectEvent};
	__property TDeviceChangeEvent OnDeviceChange={read=FDeviceChangeEvent,write=FDeviceChangeEvent};
};
extern TModem *ptrModem;

//*****************************************************************************
// TAPI callback function
//
void CALLBACK LineCallback(	DWORD hDevice, DWORD dwMsg, DWORD dwCallbackInstance,
							DWORD dwParam1, DWORD dwParam2, DWORD dwParam3);

#endif
